import {
  scaleTime,
  extent,
  histogram as bin,
  timeMonths,
  timeFormat,
  sum,
  scaleLinear,
  min,
  max,
  brushX,
  select,
  event,
} from 'd3';
import { useEffect, useRef, useMemo } from 'react';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';

const width = 960;
const margin = {
  top: 0,
  bottom: 35,
  left: 60,
  right: 30,
};

const tickOffset = 5;
const xAxisLabelOffset = 28;
const yAxisLabelOffset = -105;

const xAxisLabel = 'Month';
const yAxisLabel = 'Total Dead and Missing';

const yValue = (d) => d.total;

import { line, curve, curveCardinal } from 'd3';

export const Marks = ({
  yScale,
  xScale,
  binnedData,
  tooltipFormat,
  innerHeight,
  xValue,
}) => (
  <g className="marks">
    {binnedData.map((d) => (
      <rect
        className="mark"
        x={xScale(d.x0)}
        y={yScale(d.y)}
        width={xScale(d.x1) - xScale(d.x0)}
        height={innerHeight - yScale(d.y)}
        fill={'darkred'}
      >
        <title>{`${tooltipFormat(d.x0)}\nTotal: ${
          d.y
        }`}</title>
      </rect>
    ))}
  </g>
);

export const DateHistogram = ({
  migrants,
  height,
  setBrushExtent,
  xValue,
  xAxisTickFormat,
}) => {
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(migrants, xValue))
        .range([0, innerWidth])
        .nice(),
    [migrants, xValue, innerWidth]
  );

  const binnedData = useMemo(
    () =>
      bin()
        .value(xValue)
        .domain(xScale.domain())
        .thresholds(
          timeMonths(xScale.domain()[0], xScale.domain()[1])
        )(migrants)
        .map((array) => ({
          y: sum(array, yValue),
          x0: array.x0,
          x1: array.x1,
        })),
    [xValue, yValue, xScale, migrants]
  );

  const yScale = useMemo(
    () =>
      scaleLinear()
        .domain([0, max(binnedData, (d) => d.y)])
        .range([innerHeight, 0]),
    [binnedData, innerHeight]
  );

  const brushRef = useRef();

  const brushed = () => {
    event.selection
      ? setBrushExtent(event.selection.map(xScale.invert))
      : setBrushExtent(null);
  };

  useEffect(() => {
    const brush = brushX().extent([
      [0, 0],
      [innerWidth, innerHeight],
    ]);
    brush(select(brushRef.current));
    brush.on('brush end', brushed);
  }, [innerWidth, innerHeight]);

  return (
    <>
      <rect
        fill={'#1b195e'}
        width={width}
        height={height}
        opacity={'1'}
      />
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <rect
          fill={'white'}
          width={innerWidth}
          height={innerHeight}
          opacity={'1'}
        />
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={tickOffset}
        />
        <AxisLeft
          yScale={yScale}
          innerWidth={innerWidth}
          tickOffset={tickOffset}
        />
        <text
          x={innerWidth / 2}
          style={{ textAnchor: 'middle' }}
          y={innerHeight + xAxisLabelOffset}
          className="axis-label"
        >
          {xAxisLabel}
        </text>
        <text
          x={innerHeight + yAxisLabelOffset}
          style={{ textAnchor: 'middle' }}
          y={-innerHeight / 2 - 8}
          className="y-axis-label"
          transform="rotate(-90)"
        >
          {yAxisLabel}
        </text>
        <Marks
          xScale={xScale}
          yScale={yScale}
          binnedData={binnedData}
          tooltipFormat={xAxisTickFormat}
          innerHeight={innerHeight}
        />
        <g ref={brushRef} />
      </g>
    </>
  );
};
